import { DataEntity } from "./data-structures/data-entity.class";
import { Observable } from "rxjs/Rx";
import { DataCollection } from "./data-structures/data-collection.class";
import { LocalStorage } from "./data-interfaces/local-storage/local-storage.class";
import { Http } from "./data-interfaces/http/http.class";
import { Nodejs } from "./data-interfaces/nodejs/nodejs.class";
import { CollectionStore } from "./stores/collection-store.class";
import { EntityStore } from "./stores/entity-store.class";
import { ReplaySubject } from "rxjs/Rx";
import { Drupal8 } from "./data-interfaces/drupal8/drupal8.class";
import { combineLatest } from 'rxjs/observable/combineLatest';
import { CollectionPaginator } from "./collection-paginator.class";
import { CordovaLocal } from "./data-interfaces/cordova-local/cordova-local.class";
import { Subject } from "rxjs";
/**
 * Data connector class
 */
var DataConnector = /** @class */ (function () {
    /**
     * Create a dataConnector
     * @param {DataConnectorConfig} configuration Data connector configuration
     */
    function DataConnector(configuration) {
        var _this = this;
        this.configuration = configuration;
        /**
         * Available interfaces
         * @type {{}} External interfaces, indexed by name
         */
        this.interfaces = {};
        /**
         * Entities store
         * @type {{}} Entities stores, indexed by endpoint name
         */
        this.entitiesLiveStore = {};
        /**
         * Collections store
         * @type {{}} Collections stores, indexed by endpoint name
         */
        this.collectionsLiveStore = {};
        /**
         * Server push listeners
         * @type {{}} Listeners, indexed by endpoint name
         */
        this.pushListeners = {};
        /**
         * Built-in external interfaces
         * @type {{}}
         */
        this.builtInFactories = {
            localstorage: LocalStorage,
            http: Http,
            nodejs: Nodejs,
            drupal8: Drupal8,
            cordova: CordovaLocal
        };
        this.globalMessageSubject = new ReplaySubject(1);
        if (this.configuration.declarations) {
            for (var declarationKey in this.configuration.declarations) {
                if (this.configuration.declarations.hasOwnProperty(declarationKey)) {
                    this.builtInFactories[declarationKey] = this.builtInFactories[this.configuration.declarations[declarationKey]];
                }
            }
        }
        for (var interfaceName in configuration.configuration) {
            if (configuration.configuration.hasOwnProperty(interfaceName)) {
                this.interfaces[interfaceName] = new this.builtInFactories[interfaceName](configuration.configuration[interfaceName], this, interfaceName);
            }
        }
        this.retryTimeout = this.configuration.retryTimeout || 50000;
        this.maxRetry = this.configuration.maxRetry || 5;
        if (typeof this.configuration.language === "string") {
            this.currentLanguage = this.configuration.language;
        }
        else if (this.configuration.language instanceof Observable) {
            this.configuration.language.subscribe(function (language) {
                _this.currentLanguage = language;
            });
        }
    }
    DataConnector.prototype.sendMessage = function (error) {
        if (error === void 0) { error = null; }
        this.globalMessageSubject.next(error);
    };
    /**
     *
     * @param {string} type
     * @returns {number}
     */
    DataConnector.prototype.getRetryTimeout = function (type) {
        return this.retryTimeout;
    };
    /**
     *
     * @param {string} type
     * @returns {number}
     */
    DataConnector.prototype.getMaxRetry = function (type) {
        return this.retryTimeout;
    };
    /**
     *
     * @param {string} language
     */
    DataConnector.prototype.setLanguage = function (language) {
        if (typeof this.configuration.language === "string" || !this.configuration.language) {
            this.currentLanguage = language;
        }
    };
    /**
     *
     * @param {number} code
     */
    DataConnector.prototype.globalCallback = function (code) {
        var callbackId = "";
        this.configuration.globalCallback(callbackId);
    };
    /**
     * Get data interface by endpoint name
     * @param {string} type Endpoint name
     * @returns {ExternalInterface} External interface
     */
    DataConnector.prototype.getInterface = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (typeof conf === "string") {
            return this.interfaces[conf];
        }
        else if (conf && conf.type) {
            return this.interfaces[conf.type];
        }
        else {
            return this.interfaces[this.configuration.defaultInterface];
        }
    };
    /**
     * Get endpoint configuration
     * @param {string} type Endpoint name
     * @returns {string | EndpointConfig} Type of the endpoint, or endpoint configuration object
     */
    DataConnector.prototype.getEndpointConfiguration = function (type) {
        return this.configuration.map[type];
    };
    /**
     * Get model schema used by the endpoint
     * @param {string} type Endpoint name
     * @returns {ModelSchema} The model schema
     */
    DataConnector.prototype.getEndpointStructureModel = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return conf.structure;
        }
    };
    /**
     * Get nesting attributes types
     * @param {string} type Endpoint name
     * @returns {{[p: string]: string}} The nested attributes types
     */
    DataConnector.prototype.getNesting = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return conf.nesting || null;
        }
    };
    /**
     *
     * @param {string} type
     * @returns {{[p: string]: any}}
     */
    DataConnector.prototype.getDatas = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return conf.datas || null;
        }
    };
    /**
     *
     * @param {string} type
     * @returns {{[p: string]: string}}
     */
    DataConnector.prototype.getEmbeddings = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return conf.embeddings || null;
        }
    };
    /**
     * Is this endpoint using connector cache
     * @param {string} type Name of the endpoint
     * @returns {boolean} True if the endpoint use cache
     */
    DataConnector.prototype.useCache = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return !!conf.cached;
        }
        return false;
    };
    /**
     * Get optional keys excluded for saving entities in this endpoint
     * @param {string} type Endpoint name
     * @returns {string[]} A list of string keys
     */
    DataConnector.prototype.getExclusions = function (type) {
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object") {
            return conf.exclusions ? conf.exclusions : [];
        }
        return [];
    };
    /**
     * Get the observable associated to an entity from the store
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @returns {Observable<DataEntity>} The observable associated to the entity
     */
    DataConnector.prototype.getEntityObservableInStore = function (type, id) {
        if (this.entitiesLiveStore[type]) {
            return this.entitiesLiveStore[type].getEntityObservable(id);
        }
        return null;
    };
    /**
     * Get the observable associated to the collection from the store
     * @param {string} type Endpoint name
     * @param {FilterData} filter Filter object
     * @param useCache
     * @returns {Observable<DataCollection>} The observable associated to the collection
     */
    DataConnector.prototype.getCollectionObservableInStore = function (type, filter, useCache) {
        if (useCache === void 0) { useCache = false; }
        if (this.collectionsLiveStore[type]) {
            return this.collectionsLiveStore[type].getCollectionSubject(filter, useCache);
        }
        return null;
    };
    /**
     * Get the observable associated to an entity from the store, if the store is undefined, create it
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @returns {Observable<DataEntity>} The observable associated to the entity
     */
    DataConnector.prototype.getEntitySubject = function (type, id, createObservable) {
        if (createObservable === void 0) { createObservable = false; }
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        return this.entitiesLiveStore[type].getEntityObservable(id, createObservable);
    };
    /**
     * Register entity in the stores
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {DataEntity} entity Entity
     * @param {Observable<DataEntity>} entityObservable Observable to register
     * @returns {Observable<DataEntity>} The observable associated to the entity
     */
    DataConnector.prototype.registerEntity = function (type, id, entity, entityObservable) {
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        this.collectionsLiveStore[entity.type].registerEntityInCollections(entity, entityObservable);
        return this.entitiesLiveStore[type].registerEntity(entity, id);
    };
    DataConnector.prototype.registerEntityByData = function (type, id, entityData) {
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        var entity = new DataEntity(type, entityData, this, id);
        var obs = this.getEntitySubject(type, id);
        this.collectionsLiveStore[entity.type].registerEntityInCollections(entity, obs);
        this.entitiesLiveStore[type].registerEntity(entity, id);
        if (this.pushListeners[type]) {
            this.pushListeners[type].next(entity);
        }
    };
    /**
     *
     * @param {string} type
     * @param {DataCollection} collection
     * @returns {Observable<DataEntity>[]}
     */
    DataConnector.prototype.registerCollectionEntities = function (type, collection) {
        var _this = this;
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        var entitiesObservables = [];
        collection.entities.forEach(function (entity) {
            var entityObservable = _this.getEntitySubject(type, entity.id);
            _this.collectionsLiveStore[entity.type].registerEntityInCollections(entity, entityObservable, false);
            entitiesObservables.push(_this.entitiesLiveStore[type].registerEntity(entity, entity.id));
        });
        return entitiesObservables;
    };
    DataConnector.prototype.replaceCollectionEntities = function (type, collection, filter) {
        var _this = this;
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        this.collectionsLiveStore[type].clearEntities(filter);
        var entitiesObservables = [];
        collection.entities.forEach(function (entity) {
            var entityObservable = _this.getEntitySubject(type, entity.id);
            _this.collectionsLiveStore[entity.type].registerEntityInCollections(entity, entityObservable, false);
            entitiesObservables.push(_this.entitiesLiveStore[type].registerEntity(entity, entity.id));
        });
        return entitiesObservables;
    };
    /**
     * Associate an entity suject the the entity in the entity store
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {ReplaySubject<DataEntity>} subject Subject to associate
     */
    DataConnector.prototype.registerEntitySubject = function (type, id, subject) {
        if (!this.entitiesLiveStore[type]) {
            this.entitiesLiveStore[type] = new EntityStore();
        }
        this.entitiesLiveStore[type].registerEntitySubject(id, subject);
    };
    /**
     * Get observable associated to the collection from the store. If store is undefined, create it
     * @param {string} type Endpoint name
     * @param {FilterData} filter Filter object
     * @param useCache
     * @returns {Observable<DataCollection>} Observable associated to the collection
     */
    DataConnector.prototype.getCollectionObservable = function (type, filter, useCache) {
        if (useCache === void 0) { useCache = false; }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        return this.collectionsLiveStore[type].getCollectionSubject(filter, useCache);
    };
    /**
     * Register the collection and collection entities in the store
     * @param {string} type Endpoint name
     * @param {FilterData} filter Filter object
     * @param {DataCollection} collection Collection to register
     * @param {boolean} refresh
     * @returns {Observable<DataCollection>} The observable associated to the collection
     */
    DataConnector.prototype.registerCollection = function (type, filter, collection, refresh) {
        if (refresh === void 0) { refresh = true; }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        if (!collection.paginated) {
            collection.entitiesObservables = this.registerCollectionEntities(type, collection);
        }
        else {
            collection.entitiesObservables = this.replaceCollectionEntities(type, collection, filter);
        }
        var obs = this.collectionsLiveStore[type].registerCollection(collection, filter);
        // refresh de la collection
        if (refresh) {
            this.collectionsLiveStore[type].refreshCollections(filter);
        }
        return obs;
    };
    DataConnector.prototype.paginatedRegisterCollection = function (type, filter, collection, refresh) {
        if (refresh === void 0) { refresh = true; }
        if (!this.collectionsLiveStore[type]) {
            this.collectionsLiveStore[type] = new CollectionStore();
        }
        collection.entitiesObservables = this.registerCollectionEntities(type, collection);
        var obs = this.collectionsLiveStore[type].registerCollection(collection, filter);
        // refresh de la collection
        if (refresh) {
            this.collectionsLiveStore[type].refreshCollections(filter);
        }
        return obs;
    };
    /**
     * Authenticate to the service
     * @param {string} serviceName Name of service on which we authenticate
     * @param {string} login User login
     * @param {string} password User password
     */
    DataConnector.prototype.authenticate = function (serviceName, login, password) {
        var _this = this;
        var selectedInterface = this.interfaces[serviceName];
        var subject = new ReplaySubject(1);
        var errorHandler = function (error) {
            _this.sendMessage(error);
            subject.error(error);
        };
        selectedInterface.authenticate(login, password, errorHandler).map(function (data) {
            _this.sendMessage();
            return new DataEntity("users", data, _this, data.id);
        }).subscribe(function (entity) {
            subject.next(entity);
        });
        return subject;
    };
    /**
     *
     * @param {string} serviceName
     * @returns {Observable<DataEntity>}
     */
    DataConnector.prototype.authenticated = function (serviceName) {
        var _this = this;
        var selectedInterface = this.interfaces[serviceName];
        return selectedInterface.authenticated.map(function (data) {
            return new DataEntity("users", data, _this, data.id);
        });
    };
    /**
     *
     * @returns {Observable<boolean>}
     */
    DataConnector.prototype.logout = function (serviceName) {
        var selectedInterface = this.interfaces[serviceName];
        this.clear();
        return selectedInterface.logout();
    };
    /**
     * Release endpoint if not used
     * @param {string} type Endpoint name
     */
    DataConnector.prototype.release = function (type) {
    };
    DataConnector.prototype.clear = function () {
        this.entitiesLiveStore = {};
        this.collectionsLiveStore = {};
        this.pushListeners = {};
        for (var key in this.interfaces) {
            this.interfaces[key].clear();
        }
    };
    /**
     * Listen for an endpoint to be notified when data is pushed from the backend
     * @param {string} type Endpoint name
     * @returns {Observable<DataEntity>} DataEntity observable associated to this entity
     */
    DataConnector.prototype.listen = function (type) {
        if (!this.pushListeners[type]) {
            this.pushListeners[type] = new Subject();
        }
        return this.pushListeners[type];
    };
    /**
     * Load entity in specified endpoint
     * @param {string} type Endpoint name
     * @param {number} id Entity id
     * @returns {Observable<DataEntity>} DataEntity observable associated to this entity
     */
    DataConnector.prototype.loadEntity = function (type, id) {
        var _this = this;
        if (this.useCache(type)) {
            var obs = this.getEntityObservableInStore(type, id);
            if (obs) {
                return obs;
            }
        }
        var selectedInterface = this.getInterface(type);
        var count = 0;
        var entityData;
        if (selectedInterface) {
            var entitySubject_1 = this.getEntitySubject(type, id, true);
            var structure_1 = this.getEndpointStructureModel(type);
            var embeddings_1 = this.getEmbeddings(type);
            var checkResponse_1 = function () {
                _this.sendMessage();
                if (entityData instanceof Observable) {
                    entityData.subscribe(function (entity) {
                        if (entity) {
                            if (structure_1) {
                                entity = structure_1.filterModel(entity);
                            }
                            // hasNesting ?
                            //let nested:{[key:string]:string} = this.getNesting(type);
                            _this.registerEntity(type, id, new DataEntity(type, entity, _this, id, embeddings_1), entitySubject_1);
                        }
                    });
                }
                else {
                    if (entityData) {
                        if (structure_1) {
                            entityData = structure_1.filterModel(entityData);
                        }
                        var newEntity = new DataEntity(type, entityData, _this, id, embeddings_1);
                        // hasNesting ?
                        /*let nested:{[key:string]:string} = this.getNesting(type);

                        if (nested) {
                            let nestedKeys:string[] = Object.keys(nested);

                            nestedKeys.forEach((key:string) => {
                                let nestedObservables:Observable<DataEntity>[] = [];

                                if (entityData[key] !== undefined && typeof entityData[key] === "number") {
                                    this.loadEntity(nested[key], entityData[key]).subscribe((loadedEntity:DataEntity) => {
                                        newEntity.nesting[key] = loadedEntity;
                                    });
                                }

                                if (entityData[key] !== undefined && Array.isArray(entityData[key])) {
                                    this.loadEntities(nested[key], entityData[key]).subscribe((loadedEntities:DataEntity[]) => {
                                        newEntity.nesting[key] = loadedEntities;
                                    });
                                }
                            });
                        }*/
                        _this.registerEntity(type, id, newEntity, entitySubject_1);
                    }
                }
            };
            var errorHandler_1 = function (error) {
                var msg = "Error loading entity of type '" + type + "' with id " + id + ". Error " + error.code;
                console.warn(msg);
                error.message = msg;
                _this.sendMessage(error);
                if (error.code > 0) {
                    entitySubject_1.error(error);
                    _this.entitiesLiveStore[type].unregister(id);
                }
                else {
                    if (count < _this.getMaxRetry(type) || _this.getMaxRetry(type) === -1) {
                        setTimeout(function () {
                            entityData = selectedInterface.loadEntity(type, id, errorHandler_1);
                            checkResponse_1();
                        }, _this.getRetryTimeout(type));
                        count++;
                    }
                    else {
                        entitySubject_1.error(error);
                        _this.entitiesLiveStore[type].unregister(id);
                    }
                }
            };
            entityData = selectedInterface.loadEntity(type, id, errorHandler_1);
            checkResponse_1();
            return entitySubject_1;
        }
    };
    /**
     * Load many entities
     * @param {string} type Endpoint name
     * @param {number[]} ids Entities ids array
     * @returns {Observable<DataEntity[]>} The data entities
     */
    DataConnector.prototype.loadEntities = function (type, ids) {
        // TODO: check si une méthode loadEntities optimisée existe dans l'interface
        var _this = this;
        // TODO: Pourrait retourner un objet indexé par id plutôt qu'un Array
        var observables = [];
        ids.forEach(function (id) {
            observables.push(_this.loadEntity(type, id));
        });
        return combineLatest.apply(void 0, observables);
    };
    DataConnector.prototype.paginatedLoadCollection = function (type, options) {
        var paginator = new CollectionPaginator(this, type, options, options.filter);
        return this.paginatedLoadCollectionExec(type, options.filter || {}, paginator);
    };
    DataConnector.prototype.paginatedLoadCollectionExec = function (type, filter, paginator) {
        var _this = this;
        var selectedInterface = this.getInterface(type);
        var structure = this.getEndpointStructureModel(type);
        var count = 0;
        var embeddings = this.getEmbeddings(type);
        if (selectedInterface) {
            var collectionSubject_1 = this.getCollectionObservable(type, filter);
            var collection_1;
            var checkResponse_2 = function () {
                _this.sendMessage();
                if (collection_1 instanceof Observable) {
                    collection_1.subscribe(function (newCollection) {
                        // ici ?
                        var coll = new DataCollection(type, newCollection, _this, structure, embeddings);
                        coll.paginated = true;
                        _this.registerCollection(type, filter, coll);
                    });
                }
                else {
                    // et là ??
                    var coll = new DataCollection(type, collection_1, _this, structure, embeddings);
                    coll.paginated = true;
                    _this.registerCollection(type, filter, coll);
                }
            };
            var errorHandler_2 = function (error) {
                var msg = "Error loading collection of type '" + type + "' with data " + JSON.stringify(filter);
                console.warn(msg);
                error.message = msg;
                _this.sendMessage(error);
                if (error.code > 0) {
                    collectionSubject_1.error(error);
                    _this.collectionsLiveStore[type].unregister(filter);
                }
                else {
                    if (count < _this.getMaxRetry(type) || _this.getMaxRetry(type) === -1) {
                        setTimeout(function () {
                            collection_1 = selectedInterface.paginatedLoadCollection(type, {
                                filter: filter,
                                page: paginator.page,
                                range: paginator.range,
                                offset: paginator.offset,
                                urlExtension: paginator.urlExtension,
                                orderOptions: paginator.orderOptions
                            }, paginator, errorHandler_2);
                            checkResponse_2();
                        }, _this.getRetryTimeout(type));
                        count++;
                    }
                    else {
                        collectionSubject_1.error(error);
                        _this.collectionsLiveStore[type].unregister(filter);
                    }
                }
            };
            collection_1 = selectedInterface.paginatedLoadCollection(type, {
                filter: filter,
                page: paginator.page,
                range: paginator.range,
                offset: paginator.offset,
                urlExtension: paginator.urlExtension,
                orderOptions: paginator.orderOptions
            }, paginator, errorHandler_2);
            checkResponse_2();
            return {
                collectionObservable: collectionSubject_1,
                paginator: paginator
            };
        }
        return null;
    };
    /**
     * Load collection from specified endpoint
     * @param {string} type Endpoint name
     * @param {FilterData} filter Filter object
     * @returns {Observable<DataCollection>} Observable associated to this collection
     */
    DataConnector.prototype.loadCollection = function (type, filter) {
        var _this = this;
        if (filter === void 0) { filter = {}; }
        var useCache = this.useCache(type);
        if (useCache) {
            var obs = this.getCollectionObservableInStore(type, filter, useCache);
            if (obs) {
                return obs;
            }
        }
        var selectedInterface = this.getInterface(type);
        var structure = this.getEndpointStructureModel(type);
        var count = 0;
        var embeddings = this.getEmbeddings(type);
        if (selectedInterface) {
            var collectionSubject_2 = this.getCollectionObservable(type, filter, useCache);
            var collection_2;
            var checkResponse_3 = function () {
                _this.sendMessage();
                if (collection_2 instanceof Observable) {
                    // attention, dans le cas de nodeJs, on ne doit pas faire de take(1)
                    collection_2.subscribe(function (newCollection) {
                        _this.registerCollection(type, filter, new DataCollection(type, newCollection, _this, structure, embeddings));
                    });
                }
                else {
                    _this.registerCollection(type, filter, new DataCollection(type, collection_2, _this, structure, embeddings));
                }
            };
            var errorHandler_3 = function (error) {
                var msg = "Error loading collection of type '" + type + "' with data " + JSON.stringify(filter);
                console.warn(msg);
                error.message = msg;
                _this.sendMessage(error);
                if (error.code > 0) {
                    collectionSubject_2.error(error);
                    _this.collectionsLiveStore[type].unregister(filter);
                }
                else {
                    if (count < _this.getMaxRetry(type) || _this.getMaxRetry(type) === -1) {
                        setTimeout(function () {
                            collection_2 = selectedInterface.loadCollection(type, filter, errorHandler_3);
                            checkResponse_3();
                        }, _this.getRetryTimeout(type));
                        count++;
                    }
                    else {
                        collectionSubject_2.error(error);
                        _this.collectionsLiveStore[type].unregister(filter);
                    }
                }
            };
            collection_2 = selectedInterface.loadCollection(type, filter, errorHandler_3);
            checkResponse_3();
            return collectionSubject_2;
        }
        return null;
    };
    DataConnector.prototype.sendReloadNotification = function (type, data) {
        if (data === void 0) { data = null; }
        var conf = this.getEndpointConfiguration(type);
        // il faut supprimer les embeddings de l'objet, pour le pas surcharger le service nodejs
        if (conf && typeof conf === "object") {
            if (conf.refreshEnabled) {
                var dataToSend = {};
                var embeddings = this.getEmbeddings(type);
                for (var key in data) {
                    if (!embeddings || embeddings[key] === undefined) {
                        dataToSend[key] = data[key];
                    }
                }
                var refreshData = {
                    myType: type
                };
                if (data) {
                    refreshData["data"] = dataToSend;
                }
                this.createEntity(this.configuration.liveRefreshService, refreshData, false);
            }
        }
    };
    /**
     * Save entity
     * @param {DataEntity} entity Entity to save
     * @param {boolean} forceReload whether to reload data if nothing is saved or not
     * @param {boolean} dispatchBeforeResponse register entity before save happens
     * @returns {Observable<DataEntity>} Observable associated to the entity
     */
    DataConnector.prototype.saveEntity = function (entity, forceReload, dispatchBeforeResponse) {
        var _this = this;
        if (forceReload === void 0) { forceReload = false; }
        if (dispatchBeforeResponse === void 0) { dispatchBeforeResponse = false; }
        var selectedInterface = this.getInterface(entity.type);
        var structure = this.getEndpointStructureModel(entity.type);
        var dataToSave;
        if (selectedInterface.useDiff) {
            dataToSave = entity.getDiff();
        }
        else {
            dataToSave = entity.getClone();
        }
        var exclusions = this.getExclusions(entity.type);
        exclusions.forEach(function (key) {
            if (dataToSave[key]) {
                delete dataToSave[key];
            }
        });
        //let entitySubject:ReplaySubject<DataEntity> = this.getEntitySubject(entity.type, entity.id);
        var entitySubject = new ReplaySubject(1);
        var count = 0;
        var entityData;
        var embeddings = this.getEmbeddings(entity.type);
        if (dispatchBeforeResponse) {
            this.registerEntity(entity.type, entity.id, entity, entitySubject);
        }
        var checkResponse = function () {
            _this.sendMessage();
            if (entityData instanceof Observable) {
                entityData.subscribe(function (saveEntity) {
                    if (structure) {
                        saveEntity = structure.filterModel(saveEntity);
                    }
                    var ent = new DataEntity(entity.type, saveEntity, _this, entity.id, embeddings);
                    entitySubject.next(ent);
                    _this.registerEntity(entity.type, entity.id, ent, entitySubject);
                    _this.sendReloadNotification(entity.type, saveEntity);
                });
            }
            else {
                if (structure) {
                    entityData = structure.filterModel(entityData);
                }
                var ent = new DataEntity(entity.type, entityData, _this, entity.id, embeddings);
                entitySubject.next(ent);
                _this.registerEntity(entity.type, entity.id, ent, entitySubject);
                _this.sendReloadNotification(entity.type, entityData);
            }
        };
        var errorHandler = function (error) {
            var msg = "Error saving entity of type '" + entity.type + "' with id " + entity.id;
            console.warn(msg);
            error.message = msg;
            _this.sendMessage(error);
            if (error.code > 0) {
                entitySubject.error(error);
                _this.entitiesLiveStore[entity.type].unregister(entity.id);
            }
            else {
                if (count < _this.getMaxRetry(entity.type) || _this.getMaxRetry(entity.type) === -1) {
                    setTimeout(function () {
                        if (!selectedInterface.useDiff || Object.keys(dataToSave).length > 0) {
                            entityData = selectedInterface.saveEntity(dataToSave, entity.type, entity.id, errorHandler);
                            checkResponse();
                        }
                        else {
                        }
                    }, _this.getRetryTimeout(entity.type));
                    count++;
                }
                else {
                    entitySubject.error(error);
                    _this.entitiesLiveStore[entity.type].unregister(entity.id);
                }
            }
        };
        if (!selectedInterface.useDiff || Object.keys(dataToSave).length > 0) {
            entityData = selectedInterface.saveEntity(dataToSave, entity.type, entity.id, errorHandler);
            checkResponse();
        }
        else if (forceReload) {
            this.loadEntity(entity.type, entity.id)
                .take(1)
                .subscribe(function (newEntity) {
                entitySubject.next(newEntity);
            });
        }
        else {
            entitySubject.next(entity);
        }
        return entitySubject;
    };
    /**
     * Create entity to the specified endpoint service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @returns {Observable<DataEntity>} The observable associated to this entity
     */
    DataConnector.prototype.createEntity = function (type, data, sendNotification) {
        var _this = this;
        if (data === void 0) { data = {}; }
        if (sendNotification === void 0) { sendNotification = true; }
        var selectedInterface = this.getInterface(type);
        var structure = this.getEndpointStructureModel(type);
        if (structure) {
            data = structure.generateModel(null, data);
        }
        var exclusions = this.getExclusions(type);
        exclusions.forEach(function (key) {
            if (data[key]) {
                delete data[key];
            }
        });
        var entitySubject = new ReplaySubject(1);
        var count = 0;
        var entity;
        var embeddings = this.getEmbeddings(type);
        var checkResponse = function () {
            _this.sendMessage();
            if (entity instanceof Observable) {
                entity.subscribe(function (createdEntity) {
                    _this.registerEntitySubject(type, createdEntity.id, entitySubject);
                    _this.registerEntity(type, createdEntity.id, new DataEntity(type, createdEntity, _this, createdEntity.id, embeddings), entitySubject);
                    if (sendNotification) {
                        _this.sendReloadNotification(type, createdEntity);
                    }
                });
            }
            else {
                _this.registerEntitySubject(type, entity.id, entitySubject);
                _this.registerEntity(type, entity.id, new DataEntity(type, entity, _this, entity.id, embeddings), entitySubject);
                if (sendNotification) {
                    _this.sendReloadNotification(type, entity);
                }
            }
        };
        var errorHandler = function (error) {
            var msg = "Error creating entity of type '" + type + "'";
            console.warn(msg);
            error.message = msg;
            _this.sendMessage(error);
            if (error.code > 0) {
                entitySubject.error(error);
            }
            else {
                if (count < _this.getMaxRetry(type) || _this.getMaxRetry(type) === -1) {
                    setTimeout(function () {
                        entity = selectedInterface.createEntity(type, data, errorHandler);
                        checkResponse();
                    }, _this.getRetryTimeout(type));
                    count++;
                }
                else {
                    entitySubject.error(error);
                }
            }
        };
        entity = selectedInterface.createEntity(type, data, errorHandler);
        checkResponse();
        return entitySubject;
    };
    /**
     * Creates an entity on the front only (will be saved on the server later)
     * @param {string} type Endpoint type
     * @param {{[p: string]: any}} data Data used to create the entity
     * @returns {Observable<DataEntity>} The observable associated to this entity
     */
    DataConnector.prototype.createTemporaryEntity = function (type, data) {
        if (data === void 0) { data = {}; }
        var structure = this.getEndpointStructureModel(type);
        if (structure) {
            data = structure.generateModel(null, data);
        }
        var entitySubject = new ReplaySubject(1);
        var entity = new DataEntity(type, data, this, -1);
        // pas utile
        //entitySubject.next(entity);
        // attention, pas d'id, car pas de retour du serveur
        this.registerEntitySubject(type, null, entitySubject);
        this.registerEntity(type, null, entity, entitySubject);
        return entitySubject;
    };
    /**
     * Delete an entity
     * @param {DataEntity} entity Entity to delete
     * @returns {Observable<boolean>} True if deletion success
     */
    DataConnector.prototype.deleteEntity = function (entity) {
        var _this = this;
        var selectedInterface = this.getInterface(entity.type);
        var subject = new ReplaySubject(1);
        var count = 0;
        var checkResponse = function () {
            _this.sendMessage();
            var cloned = Object.assign({}, entity.attributes);
            cloned['id'] = entity.id;
            if (result instanceof Observable) {
                result.subscribe(function (res) {
                    _this.unregisterEntity(entity);
                    subject.next(res);
                    _this.sendReloadNotification(entity.type, entity.attributes);
                });
            }
            else {
                _this.unregisterEntity(entity);
                subject.next(result);
                _this.sendReloadNotification(entity.type, entity.attributes);
            }
        };
        var result;
        var errorHandler = function (error) {
            var msg = "Error deleting entity of type '" + entity.type + "' with id " + entity.id;
            console.warn(msg);
            error.message = msg;
            _this.sendMessage(error);
            if (error.code > 0) {
                subject.error(error);
                _this.entitiesLiveStore[entity.type].unregister(entity.id);
            }
            else {
                if (count < _this.getMaxRetry(entity.type) || _this.getMaxRetry(entity.type) === -1) {
                    setTimeout(function () {
                        result = selectedInterface.deleteEntity(entity.type, entity.id, errorHandler);
                        checkResponse();
                    }, _this.getRetryTimeout(entity.type));
                    count++;
                }
                else {
                    subject.error(error);
                    _this.entitiesLiveStore[entity.type].unregister(entity.id);
                }
            }
        };
        result = selectedInterface.deleteEntity(entity.type, entity.id, errorHandler);
        checkResponse();
        return subject;
    };
    /**
     * Delete entity from store
     * @param {DataEntity} entity Entity to delete
     */
    DataConnector.prototype.unregisterEntity = function (entity) {
        if (this.collectionsLiveStore[entity.type]) {
            this.collectionsLiveStore[entity.type].deleteEntityFromCollection(entity);
        }
        if (this.entitiesLiveStore[entity.type]) {
            this.entitiesLiveStore[entity.type].unregisterEntity(entity);
        }
    };
    /**
     *
     * @param {string} type
     * @param {number | string} id
     */
    DataConnector.prototype.unregisterEntityTypeAndId = function (type, id) {
        var _this = this;
        this.entitiesLiveStore[type].getEntityObservable(id).take(1).subscribe(function (entity) {
            _this.unregisterEntity(entity);
        });
    };
    /**
     * Refresh entity (from refresh service)
     * @param {string} type Endpoint name
     * @param {number} id Entity id
     */
    DataConnector.prototype.refreshEntity = function (type, id) {
        var selectedInterface = this.getInterface(type);
        if (this.entitiesLiveStore[type] && this.entitiesLiveStore[type].isInStore(id)) {
            this.loadEntity(type, id);
        }
    };
    /**
     * Refresh collection (from refresh service)
     * @param {string} type Endpoint name
     * @param {FilterData} filter Collection filter object
     */
    DataConnector.prototype.refreshCollection = function (type, filter) {
        // let selectedInterface:ExternalInterface = this.getInterface(type);
        if (this.collectionsLiveStore[type] && this.collectionsLiveStore[type].isInStore(filter)) {
            this.loadCollection(type, filter);
        }
    };
    DataConnector.prototype.objectMatchFilter = function (object, filter) {
        var filterKeys = Object.keys(filter);
        for (var _i = 0, filterKeys_1 = filterKeys; _i < filterKeys_1.length; _i++) {
            var key = filterKeys_1[_i];
            if (object[key] !== undefined && filter[key] !== object[key]) {
                return false;
            }
        }
        return true;
    };
    DataConnector.prototype.refreshCollectionWithData = function (type, data) {
        var store = this.collectionsLiveStore[type];
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object" && store) {
            if (conf.refreshEnabled) {
                for (var key in store.collections) {
                    var filter = store.filters[key];
                    if (this.objectMatchFilter(data, filter)) {
                        this.loadCollection(type, filter);
                    }
                }
            }
        }
    };
    DataConnector.prototype.refreshAllCollectionsOfType = function (type) {
        var store = this.collectionsLiveStore[type];
        var conf = this.getEndpointConfiguration(type);
        if (conf && typeof conf === "object" && store) {
            if (conf.refreshEnabled) {
                for (var key in store.collections) {
                    var filter = store.filters[key];
                    this.loadCollection(type, filter);
                }
            }
        }
    };
    DataConnector.prototype.getUnexpectedLogoutSubject = function (serviceName) {
        var selectedInterface = this.interfaces[serviceName];
        return selectedInterface.unexpectedLogoutSubject;
    };
    return DataConnector;
}());
export { DataConnector };
//# sourceMappingURL=data-connector.class.js.map