/**
 * Created by Christophe on 22/11/2017.
 */
import { Observable } from "rxjs/Rx";
import { CollectionDataSet, EntityDataSet, FilterData } from "../types";
import { CollectionOptionsInterface } from "../collection-options.interface";
import { CollectionPaginator } from "../collection-paginator.class";
import { Subject } from "rxjs";
/**
 * Base external interface
 */
export declare abstract class ExternalInterface {
    authenticated: Observable<EntityDataSet>;
    unexpectedLogoutSubject: Subject<null>;
    /**
     * if true, the save method will only send the modified properties to the service
     * @type {boolean}
     */
    useDiff: boolean;
    retryTimeout: number;
    maxRetry: number;
    /**
     * Load an entity from the service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} A set of data, or an observable
     */
    loadEntity(type: string, id: number | string, errorHandler?: Function): EntityDataSet | Observable<EntityDataSet>;
    /**
     * Load an entity collection from the service
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Collection filter object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {CollectionDataSet | Observable<CollectionDataSet>} A collection set of data, or an observable
     */
    loadCollection(type: string, filter: FilterData, errorHandler?: Function): CollectionDataSet | Observable<CollectionDataSet>;
    /**
     *
     */
    clear(): void;
    /**
     *
     * @param {string} type
     * @param {CollectionOptionsInterface} options
     * @returns {CollectionDataSet | Observable<CollectionDataSet>}
     */
    paginatedLoadCollection(type: string, options: CollectionOptionsInterface, paginator: CollectionPaginator, errorHandler?: Function): CollectionDataSet | Observable<CollectionDataSet>;
    /**
     * Create an entity on the service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Base data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} A set of data, or an observable
     */
    createEntity(type: string, data: EntityDataSet, errorHandler?: Function): EntityDataSet | Observable<EntityDataSet>;
    /**
     * Delete an entity from the service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {boolean | Observable<boolean>} True if deletion success
     */
    deleteEntity(type: string, id: number | string, errorHandler?: Function): boolean | Observable<boolean>;
    /**
     * Save an entity on the service
     * @param {EntityDataSet} data Data to Save
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} The saved data
     */
    saveEntity(data: EntityDataSet, type: string, id: number | string, errorHandler?: Function): EntityDataSet | Observable<EntityDataSet>;
    /**
     * Authenticating to the service
     * @param {string} login User login
     * @param {string} password User password
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>}
     */
    authenticate(login: string, password: string, errorHandler?: Function): Observable<EntityDataSet>;
    logout(): Observable<boolean>;
    /**
     * Release an endpoint if not useful anymore
     * @param {string} type Name of the endpoint
     */
    release(type: string): void;
    /**
     * Sends an error message
     * @param {number} code Error code
     * @param {string} originalMessage Error original text message
     * @param {Function} errorHandler Error handler Function
     */
    sendError(code: number, originalMessage: string, errorHandler: Function, data?: Object): void;
}
