import { ExternalInterface } from "../abstract-external-interface.class";
import { DataConnector } from "../../data-connector.class";
import { Observable } from "rxjs/Rx";
import { CollectionDataSet, EntityDataSet } from "../../types";
import { Drupal8Configuration } from "./drupal8-configuration.interface";
/**
 * Http external interface
 */
export declare class Drupal8 extends ExternalInterface {
    private configuration;
    private connector;
    private interfaceName;
    /**
     *
     * @type {boolean}
     */
    /**
     *
     */
    private dataStore;
    private headers;
    /**
     * Creates the http interface
     * @param {HttpConfiguration} configuration Configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    constructor(configuration: Drupal8Configuration, connector: DataConnector, interfaceName: string);
    /**
     * Is the user authenticated on this service ?
     * @returns {Observable<EntityDataSet>}
     */
    readonly authenticated: Observable<EntityDataSet>;
    /**
     * Add headers to the request
     * @param {XMLHttpRequest} request A xhr request
     */
    private addHeaders(request);
    /**
     * Load entity in http service
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the data
     */
    loadEntity(type: string, id: number, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     * Load a collection in http service
     * @param {string} type Endpoint name
     * @param {{[p: string]: any}} filter Filter Object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<CollectionDataSet>} Observable returning the collection data
     */
    loadCollection(type: string, filter?: {
        [key: string]: any;
    }, errorHandler?: Function): Observable<CollectionDataSet>;
    /**
     * Save entity to the http service
     * @param {EntityDataSet} entity Entity data to save
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    saveEntity(entity: EntityDataSet, type: string, id: number, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     * Create entity in http service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    createEntity(type: string, data: EntityDataSet, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     * Delete entity from http service
     * @param {string} type Endpoint type
     * @param {number} id Entity id
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if deletion success
     */
    deleteEntity(type: string, id: number, errorHandler?: Function): Observable<boolean>;
    getOauthToken(login: string, password: string): Observable<Object>;
    /**
     * Authenticate in service
     * @param {string} login User login
     * @param {string} password User password
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if authentication success
     */
    authenticate(login: string, password: string, errorHandler?: Function): Observable<EntityDataSet>;
    logOut(): void;
    /**
     *
     * @param {string} accessToken
     * @param {Function} errorHandler
     */
    private setToken(accessToken, errorHandler?);
    /**
     *
     * @param {number} expire
     */
    private setExpireDate(expire);
    /**
     *
     * @param {string} refreshToken
     * @param {Function} errorHandler
     */
    private refreshToken(refreshToken, errorHandler);
    /**
     *
     * @param {string} refreshToken
     */
    private setRefreshToken(refreshToken);
    /**
     *
     * @param {boolean} complete
     * @param {Function} errorHandler
     * @returns {Observable<EntityDataSet>}
     */
    getMe(complete?: boolean, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     *
     * @param {EntityDataSet} userData
     * @param {boolean} complete
     */
    setMe(userData: EntityDataSet, complete?: boolean): void;
    /**
     * Extract entity data from raw data
     * @param {string} responseText Response text from server
     * @returns {EntityDataSet} Entity data
     */
    protected extractEntity(responseText: string): EntityDataSet;
    /**
     * Extract collection data from raw data
     * @param {string} responseText Response text from server
     * @returns {CollectionDataSet} Collection data
     */
    protected extractCollection(responseText: string): CollectionDataSet;
}
