import { LocalStorageConfiguration } from "./local-storage-configuration.interface";
import { DataConnector } from "../../data-connector.class";
import { ExternalInterface } from "../abstract-external-interface.class";
import { CollectionDataSet, EntityDataSet, FilterData } from "../../types";
/**
 * Local storage data interface
 */
export declare class LocalStorage extends ExternalInterface {
    private configuration;
    private connector;
    /**
     * Where are stored the entities
     * @type {CollectionDataSet}
     */
    private dataStore;
    /**
     * Create a local storage data interface
     * @param {LocalStorageConfiguration} configuration Configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    constructor(configuration: LocalStorageConfiguration, connector: DataConnector);
    /**
     *
     */
    clear(): void;
    /**
     * If a prefix is defined in the configuration, returns a concatenation of prefix and endpoint name
     * @param {string} type Name of the endpoint
     * @returns {string} The prefixed endpoint name
     */
    private getPrefixedType(type);
    /**
     * Load an endpoint from the local storage
     * @param {string} pointName The endpoint name
     */
    private loadPointFromStorage(pointName);
    /**
     * Load an endpoint from the local storage if storage object is empty for that name
     * @param {string} type Name of the endpoint
     */
    private loadPointFromStorageIfEmpty(type);
    /**
     * Set entity data in local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {EntityDataSet} data Entity Data
     */
    private setEntityInStore(type, id, data);
    /**
     * Get entity data from local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} Data of the entity
     */
    private getEntityFromStore(type, id);
    /**
     * Delete entity from local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {boolean} True if deletion success
     */
    private deleteEntityFromStore(type, id);
    /**
     * Get collection data from local store
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Filter data
     * @returns {CollectionDataSet} Collection data
     */
    private getCollectionFromStore(type, filter?);
    /**
     * Save local store to localStorage, for a specified endpoint
     * @param {string} type Name of the endpoint
     */
    private savePointToStorage(type);
    /**
     * Get last used id from localStorage
     * @returns {number} The value
     */
    /**
     * Set last used id to localStorage
     * @param {number} value The value to set to
     */
    private lastUsedId;
    /**
     * Load an entity from local storage service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} The entity raw datas
     */
    loadEntity(type: string, id: number): EntityDataSet;
    /**
     * Load a collection from local storage service
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Filter data
     * @returns {CollectionDataSet} The collection raw data
     */
    loadCollection(type: string, filter?: FilterData): CollectionDataSet;
    /**
     * Save an entity to the local storage service
     * @param {EntityDataSet} entity Entity data to save
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} Saved raw data
     */
    saveEntity(entity: EntityDataSet, type: string, id: number): EntityDataSet;
    /**
     * Create an entity on the local storage service
     * @param {string} type Name of the endpoint
     * @param {EntityDataSet} data Entity Data
     * @returns {EntityDataSet} The saved raw dats
     */
    createEntity(type: string, data: EntityDataSet): EntityDataSet;
    /**
     * Delete an entity from the local storage service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {boolean} True if deletion success
     */
    deleteEntity(type: string, id: number): boolean;
}
