var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ExternalInterface } from "../abstract-external-interface.class";
/**
 * Local storage data interface
 */
var LocalStorage = /** @class */ (function (_super) {
    __extends(LocalStorage, _super);
    /**
     * Create a local storage data interface
     * @param {LocalStorageConfiguration} configuration Configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    function LocalStorage(configuration, connector) {
        var _this = _super.call(this) || this;
        _this.configuration = configuration;
        _this.connector = connector;
        /**
         * Where are stored the entities
         * @type {CollectionDataSet}
         */
        _this.dataStore = {};
        _this.useDiff = false;
        return _this;
    }
    /**
     *
     */
    LocalStorage.prototype.clear = function () {
        this.dataStore = {};
    };
    /**
     * If a prefix is defined in the configuration, returns a concatenation of prefix and endpoint name
     * @param {string} type Name of the endpoint
     * @returns {string} The prefixed endpoint name
     */
    LocalStorage.prototype.getPrefixedType = function (type) {
        if (this.configuration.prefix) {
            return this.configuration.prefix + "-" + type;
        }
        else {
            return type;
        }
    };
    /**
     * Load an endpoint from the local storage
     * @param {string} pointName The endpoint name
     */
    LocalStorage.prototype.loadPointFromStorage = function (pointName) {
        if (!localStorage[pointName] || localStorage[pointName] === "") {
            this.dataStore[pointName] = {};
        }
        else {
            this.dataStore[pointName] = JSON.parse(localStorage[pointName]);
        }
    };
    /**
     * Load an endpoint from the local storage if storage object is empty for that name
     * @param {string} type Name of the endpoint
     */
    LocalStorage.prototype.loadPointFromStorageIfEmpty = function (type) {
        var pointName = this.getPrefixedType(type);
        if (!this.dataStore[pointName]) {
            this.loadPointFromStorage(pointName);
        }
    };
    /**
     * Set entity data in local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {EntityDataSet} data Entity Data
     */
    LocalStorage.prototype.setEntityInStore = function (type, id, data) {
        var pointName = this.getPrefixedType(type);
        this.loadPointFromStorageIfEmpty(type);
        this.dataStore[pointName][id] = data;
        this.savePointToStorage(type);
    };
    /**
     * Get entity data from local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} Data of the entity
     */
    LocalStorage.prototype.getEntityFromStore = function (type, id) {
        var pointName = this.getPrefixedType(type);
        this.loadPointFromStorageIfEmpty(type);
        return this.dataStore[pointName][id];
    };
    /**
     * Delete entity from local store
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {boolean} True if deletion success
     */
    LocalStorage.prototype.deleteEntityFromStore = function (type, id) {
        var pointName = this.getPrefixedType(type);
        this.loadPointFromStorageIfEmpty(type);
        if (this.dataStore[pointName][id]) {
            delete this.dataStore[pointName][id];
            this.savePointToStorage(type);
            return true;
        }
        return false;
    };
    /**
     * Get collection data from local store
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Filter data
     * @returns {CollectionDataSet} Collection data
     */
    LocalStorage.prototype.getCollectionFromStore = function (type, filter) {
        var _this = this;
        if (filter === void 0) { filter = {}; }
        var pointName = this.getPrefixedType(type);
        this.loadPointFromStorageIfEmpty(type);
        var dataSet = {};
        var keys = Object.keys(this.dataStore[pointName]);
        var filterKeys = Object.keys(filter);
        keys.forEach(function (key) {
            var matching = true;
            filterKeys.forEach(function (filterKey) {
                if (filter[filterKey] !== _this.dataStore[pointName][+key][filterKey]) {
                    matching = false;
                }
            });
            if (matching) {
                dataSet[+key] = _this.dataStore[pointName][+key];
            }
        });
        return dataSet;
    };
    /**
     * Save local store to localStorage, for a specified endpoint
     * @param {string} type Name of the endpoint
     */
    LocalStorage.prototype.savePointToStorage = function (type) {
        var pointName = this.getPrefixedType(type);
        if (this.dataStore[pointName]) {
            localStorage[pointName] = JSON.stringify(this.dataStore[pointName]);
        }
    };
    Object.defineProperty(LocalStorage.prototype, "lastUsedId", {
        /**
         * Get last used id from localStorage
         * @returns {number} The value
         */
        get: function () {
            var lastUsedIdKey = this.getPrefixedType("lastusedid");
            if (localStorage[lastUsedIdKey] === undefined || localStorage[lastUsedIdKey] === "") {
                return 0;
            }
            else {
                return +localStorage[lastUsedIdKey];
            }
        },
        /**
         * Set last used id to localStorage
         * @param {number} value The value to set to
         */
        set: function (value) {
            var lastUsedIdKey = this.getPrefixedType("lastusedid");
            localStorage[lastUsedIdKey] = value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Load an entity from local storage service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} The entity raw datas
     */
    LocalStorage.prototype.loadEntity = function (type, id) {
        this.loadPointFromStorageIfEmpty(type);
        var data = this.getEntityFromStore(type, id);
        return data ? data : null;
    };
    /**
     * Load a collection from local storage service
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Filter data
     * @returns {CollectionDataSet} The collection raw data
     */
    LocalStorage.prototype.loadCollection = function (type, filter) {
        if (filter === void 0) { filter = {}; }
        this.loadPointFromStorageIfEmpty(type);
        var data = this.getCollectionFromStore(type, filter);
        return data ? data : null;
    };
    /**
     * Save an entity to the local storage service
     * @param {EntityDataSet} entity Entity data to save
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {EntityDataSet} Saved raw data
     */
    LocalStorage.prototype.saveEntity = function (entity, type, id) {
        this.loadPointFromStorageIfEmpty(type);
        this.setEntityInStore(type, id, entity);
        return entity;
    };
    /**
     * Create an entity on the local storage service
     * @param {string} type Name of the endpoint
     * @param {EntityDataSet} data Entity Data
     * @returns {EntityDataSet} The saved raw dats
     */
    LocalStorage.prototype.createEntity = function (type, data) {
        var newId = ++this.lastUsedId;
        data.id = newId;
        this.setEntityInStore(type, newId, data);
        return data;
    };
    /**
     * Delete an entity from the local storage service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @returns {boolean} True if deletion success
     */
    LocalStorage.prototype.deleteEntity = function (type, id) {
        this.loadPointFromStorageIfEmpty(type);
        return this.deleteEntityFromStore(type, id);
    };
    return LocalStorage;
}(ExternalInterface));
export { LocalStorage };
//# sourceMappingURL=local-storage.class.js.map