/**
 * Data entity unit object
 */
var DataEntity = /** @class */ (function () {
    /**
     * Create the data entity
     * @param {string} type Type of the entity
     * @param {EntityDataSet} data Entity data
     * @param {DataConnector} connector Reference to the connector
     * @param {number} id Entity id
     * @param {Object} embeddingsConf
     */
    function DataEntity(type, data, connector, id, embeddingsConf) {
        if (connector === void 0) { connector = null; }
        if (id === void 0) { id = null; }
        if (embeddingsConf === void 0) { embeddingsConf = null; }
        var _this = this;
        this.type = type;
        this.connector = connector;
        this.id = id;
        this.embeddingsConf = embeddingsConf;
        /**
         * Entity attributes
         * @type {{}}
         */
        this.attributes = {};
        /**
         * Nested entities
         * @type {{}}
         */
        this.nesting = {};
        //embeddings: {[key: string]: DataEntity[]} = {};
        this.relationship = {};
        /**
         *
         * @type {{}}
         */
        this.embeddings = {};
        var _loop_1 = function (key) {
            if (data.hasOwnProperty(key) && key !== "id") {
                this_1.attributes[key] = data[key];
                if (embeddingsConf && embeddingsConf[key] !== undefined && this_1.attributes[key]) {
                    if (Array.isArray(this_1.attributes[key])) {
                        this_1.embeddings[key] = [];
                        this_1.attributes[key].forEach(function (elem) {
                            _this.embeddings[key].push(new DataEntity(embeddingsConf[key], elem, connector, elem["id"], embeddingsConf));
                        });
                    }
                    else {
                        this_1.embeddings[key] = new DataEntity(embeddingsConf[key], this_1.attributes[key], connector, this_1.attributes[key]["id"], embeddingsConf);
                    }
                }
            }
        };
        var this_1 = this;
        for (var key in data) {
            _loop_1(key);
        }
        if (data["id"]) {
            this.id = data["id"];
        }
        this.generateReferenceObject();
    }
    /**
     * Set an attribute by key
     * @param {string} key Key name
     * @param value New value
     */
    DataEntity.prototype.set = function (key, value) {
        this.attributes[key] = value;
    };
    /**
     * Get an attribute by key
     * @param {string} key Key name
     * @returns {any} Value
     */
    DataEntity.prototype.get = function (key) {
        return this.attributes[key];
    };
    Object.defineProperty(DataEntity.prototype, "hasChanges", {
        /**
         *
         * @returns {boolean}
         */
        get: function () {
            if (Object.keys(this.attributes).length === 0) {
                return false;
            }
            for (var key in this.attributes) {
                if (this.attributes[key] !== this.attributesRef[key]) {
                    return true;
                }
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    /**
     *
     * @param {string} name
     * @returns {DataEntity | DataEntity[]}
     */
    DataEntity.prototype.getEmbed = function (name) {
        return this.embeddings[name];
    };
    /**
     * Save the entity
     * @returns {Observable<DataEntity>} The observable associated to the entity in connector stores
     */
    DataEntity.prototype.save = function (forceReload, dispatchBeforeResponse) {
        if (forceReload === void 0) { forceReload = false; }
        if (dispatchBeforeResponse === void 0) { dispatchBeforeResponse = false; }
        return this.saveAction(forceReload, dispatchBeforeResponse);
    };
    DataEntity.prototype.saveAction = function (forceReload, dispatchBeforeResponse) {
        var _this = this;
        if (forceReload === void 0) { forceReload = false; }
        if (dispatchBeforeResponse === void 0) { dispatchBeforeResponse = false; }
        var obs;
        if (this.id !== -1) {
            obs = this.connector.saveEntity(this, forceReload, dispatchBeforeResponse);
        }
        else {
            // temporary entity deletion
            this.remove();
            obs = this.connector.createEntity(this.type, this.attributes);
        }
        obs.take(1).subscribe(function () {
            _this.generateReferenceObject();
        });
        return obs;
    };
    /**
     * Delete the entity
     * @returns {Observable<boolean>} True if deletion success
     */
    DataEntity.prototype.remove = function () {
        return this.connector.deleteEntity(this);
    };
    /**
     * Copy the attributes to generate the new reference object (for diff)
     */
    DataEntity.prototype.generateReferenceObject = function () {
        var _this = this;
        var ref = {};
        var keys = Object.keys(this.attributes);
        keys.forEach(function (key) {
            ref[key] = _this.attributes[key];
        });
        this.attributesRef = ref;
    };
    /**
     * Get an attributes cloned object
     * @returns {EntityDataSet} The cloned attributes object
     */
    DataEntity.prototype.getClone = function () {
        var _this = this;
        var clone = {};
        var keys = Object.keys(this.attributes);
        keys.forEach(function (key) {
            clone[key] = _this.attributes[key];
        });
        return clone;
    };
    /**
     * Return the diff (only updated properties since last save action)
     * @returns {EntityDataSet} Diff object
     */
    DataEntity.prototype.getDiff = function () {
        var _this = this;
        var diff = {};
        var keys = Object.keys(this.attributes);
        keys.forEach(function (key) {
            if (_this.attributes[key] !== undefined &&
                _this.attributesRef[key] !== _this.attributes[key]) {
                diff[key] = _this.attributes[key];
            }
        });
        return diff;
    };
    return DataEntity;
}());
export { DataEntity };
//# sourceMappingURL=data-entity.class.js.map