import { Observable } from "rxjs/Observable";
import { DataCollection } from "../data-structures/data-collection.class";
import { DataEntity } from "../data-structures/data-entity.class";
import { FilterData } from "../types";
import { Subject } from "rxjs";
/**
 * Collection store: where the collections and the collection observables are stored for an endpoint
 */
export declare class CollectionStore {
    /**
     * Stored collection observables, indexed by filter hash
     * @type {{}}
     */
    private collectionObservables;
    /**
     * Filters indexed by their hash
     * @type {{}}
     */
    filters: FilterData;
    /**
     * Stored collections, indexed by filter hash
     * @type {{}}
     */
    collections: {
        [key: string]: DataCollection;
    };
    /**
     * Creates the store
     */
    constructor();
    /**
     *
     */
    clearEntities(filter: FilterData): void;
    /**
     * Registers the collection in store and update associated subject. If the subject does not exists, creates it
     * @param {DataCollection} collection Collection to register
     * @param {FilterData} filter Collection filter
     * @returns {Observable<DataCollection>} Observable associated to the collection
     */
    registerCollection(collection: DataCollection, filter: FilterData): Observable<DataCollection>;
    /**
     * Register entity in collection in store if entity match collection filter
     * @param {DataEntity} entity Entity to register
     * @param {Observable<DataEntity>} entityObservable Entity observable to register
     * @param {boolean} refreshCollection If true, the collection observable is refreshed
     */
    registerEntityInCollections(entity: DataEntity, entityObservable: Observable<DataEntity>, refreshCollection?: boolean): void;
    /**
     * Refresh collection observable by filter
     * @param {FilterData} filter Filter object
     */
    refreshCollections(filter: FilterData): void;
    /**
     * Delete entity from stored collections
     * @param {DataEntity} entity Entity to delete
     */
    deleteEntityFromCollection(entity: DataEntity): void;
    /**
     * Delete all data for a specific filter
     * @param {FilterData} filter The filter used to delete data
     */
    unregister(filter: FilterData): void;
    /**
     * Test if the entity matches the filter
     * @param {DataEntity} entity Entity to test
     * @param {FilterData} filter Filter object
     * @returns {boolean} True if the entity matches the filter
     */
    entityMatchFilter(entity: DataEntity, filter: FilterData): boolean;
    /**
     *
     * @param {FilterData} filter1
     * @param {FilterData} filter2
     * @returns {boolean}
     */
    filterMatching(filter1?: FilterData, filter2?: FilterData): boolean;
    /**
     * Returns the observable associated to the specified filter
     * @param {FilterData} filter Filter object
     * @param useCache
     * @returns {Observable<DataCollection>} The observable associated to the filter object
     */
    getCollectionSubject(filter: FilterData, useCache?: boolean): Subject<DataCollection>;
    /**
     * Returns true if collection is defined in store
     * @param {FilterData} filter Collection filter object
     * @returns {boolean}
     */
    isInStore(filter: FilterData): boolean;
}
