import * as ObjectHash from "object-hash";
import { ReplaySubject, Subject } from "rxjs";
/**
 * Collection store: where the collections and the collection observables are stored for an endpoint
 */
var CollectionStore = /** @class */ (function () {
    /**
     * Creates the store
     */
    function CollectionStore() {
        /**
         * Stored collection observables, indexed by filter hash
         * @type {{}}
         */
        this.collectionObservables = {};
        /**
         * Filters indexed by their hash
         * @type {{}}
         */
        this.filters = {};
        /**
         * Stored collections, indexed by filter hash
         * @type {{}}
         */
        this.collections = {};
    }
    /**
     *
     */
    CollectionStore.prototype.clearEntities = function (filter) {
        var hash = ObjectHash(filter);
        if (this.collections[hash]) {
            this.collections[hash].entities.length = 0;
        }
    };
    /**
     * Registers the collection in store and update associated subject. If the subject does not exists, creates it
     * @param {DataCollection} collection Collection to register
     * @param {FilterData} filter Collection filter
     * @returns {Observable<DataCollection>} Observable associated to the collection
     */
    CollectionStore.prototype.registerCollection = function (collection, filter) {
        var hash = ObjectHash(filter);
        this.filters[hash] = filter;
        this.collections[hash] = collection;
        var collectionSubject;
        if (this.collectionObservables[hash]) {
            collectionSubject = this.collectionObservables[hash];
        }
        else {
            collectionSubject = new Subject();
            this.collectionObservables[hash] = collectionSubject;
        }
        //collectionSubject.next(collection);
        return collectionSubject;
    };
    /**
     * Register entity in collection in store if entity match collection filter
     * @param {DataEntity} entity Entity to register
     * @param {Observable<DataEntity>} entityObservable Entity observable to register
     * @param {boolean} refreshCollection If true, the collection observable is refreshed
     */
    CollectionStore.prototype.registerEntityInCollections = function (entity, entityObservable, refreshCollection) {
        var _this = this;
        if (refreshCollection === void 0) { refreshCollection = true; }
        var collectionKeys = Object.keys(this.collections);
        collectionKeys.forEach(function (key) {
            if (_this.entityMatchFilter(entity, _this.filters[key])) {
                _this.collections[key].registerEntity(entity, entityObservable);
                if (refreshCollection) {
                    _this.collectionObservables[key].next(_this.collections[key]);
                }
            }
        });
    };
    /**
     * Refresh collection observable by filter
     * @param {FilterData} filter Filter object
     */
    CollectionStore.prototype.refreshCollections = function (filter) {
        var _this = this;
        var filterKeys = Object.keys(this.collectionObservables);
        filterKeys.forEach(function (hash) {
            if (_this.filterMatching(filter, _this.filters[hash])) {
                _this.collectionObservables[hash].next(_this.collections[hash]);
            }
        });
    };
    /**
     * Delete entity from stored collections
     * @param {DataEntity} entity Entity to delete
     */
    CollectionStore.prototype.deleteEntityFromCollection = function (entity) {
        var _this = this;
        var collectionKeys = Object.keys(this.collections);
        collectionKeys.forEach(function (key) {
            if (_this.entityMatchFilter(entity, _this.filters[key])) {
                _this.collections[key].deleteEntity(entity.id);
                _this.collectionObservables[key].next(_this.collections[key]);
            }
        });
    };
    /**
     * Delete all data for a specific filter
     * @param {FilterData} filter The filter used to delete data
     */
    CollectionStore.prototype.unregister = function (filter) {
        var hash = ObjectHash(filter);
        if (this.collections[hash]) {
            delete this.collections[hash];
        }
        if (this.collectionObservables[hash]) {
            delete this.collectionObservables[hash];
        }
        if (this.filters[hash]) {
            delete this.filters[hash];
        }
    };
    /**
     * Test if the entity matches the filter
     * @param {DataEntity} entity Entity to test
     * @param {FilterData} filter Filter object
     * @returns {boolean} True if the entity matches the filter
     */
    CollectionStore.prototype.entityMatchFilter = function (entity, filter) {
        var filterKeys = Object.keys(filter);
        for (var _i = 0, filterKeys_1 = filterKeys; _i < filterKeys_1.length; _i++) {
            var key = filterKeys_1[_i];
            if (entity.attributes[key] !== undefined && filter[key] !== entity.attributes[key]) {
                return false;
            }
        }
        return true;
    };
    /**
     *
     * @param {FilterData} filter1
     * @param {FilterData} filter2
     * @returns {boolean}
     */
    CollectionStore.prototype.filterMatching = function (filter1, filter2) {
        if (filter1 === void 0) { filter1 = {}; }
        if (filter2 === void 0) { filter2 = {}; }
        // must have the same keys
        var filter1Keys = Object.keys(filter1);
        var filter2Keys = Object.keys(filter2);
        if (filter1Keys.length !== filter2Keys.length) {
            return false;
        }
        for (var _i = 0, filter1Keys_1 = filter1Keys; _i < filter1Keys_1.length; _i++) {
            var key = filter1Keys_1[_i];
            if (filter2Keys.indexOf(key) === -1) {
                return false;
            }
        }
        for (var _a = 0, filter1Keys_2 = filter1Keys; _a < filter1Keys_2.length; _a++) {
            var key = filter1Keys_2[_a];
            if (filter1[key] !== filter2[key]) {
                return false;
            }
        }
        return true;
    };
    /**
     * Returns the observable associated to the specified filter
     * @param {FilterData} filter Filter object
     * @param useCache
     * @returns {Observable<DataCollection>} The observable associated to the filter object
     */
    CollectionStore.prototype.getCollectionSubject = function (filter, useCache) {
        if (useCache === void 0) { useCache = false; }
        var hash = ObjectHash(filter);
        if (this.collectionObservables[hash]) {
            return this.collectionObservables[hash];
        }
        else {
            var subject = void 0;
            if (useCache) {
                subject = new ReplaySubject();
            }
            else {
                subject = new Subject();
            }
            this.collectionObservables[hash] = subject;
            return subject;
        }
    };
    /**
     * Returns true if collection is defined in store
     * @param {FilterData} filter Collection filter object
     * @returns {boolean}
     */
    CollectionStore.prototype.isInStore = function (filter) {
        var hash = ObjectHash(filter);
        return !!this.collections[hash];
    };
    return CollectionStore;
}());
export { CollectionStore };
//# sourceMappingURL=collection-store.class.js.map