import { ReplaySubject } from "rxjs/Rx";
/**
 * Entity store: where the entities and the entities observables are stored for an endpoint
 */
var EntityStore = /** @class */ (function () {
    /**
     * Creates the store
     */
    function EntityStore() {
        /**
         * Stores entities subjects, indexed by entity id
         * @type {{}}
         */
        this.entitiesObservables = {};
    }
    EntityStore.prototype.clear = function () {
    };
    /**
     * Registers the entity in store and update associated subject. If the subject does not exists, creates it
     * @param {DataEntity} entity Entity to register
     * @param {number} id Entity id
     * @returns {Observable<DataEntity>} Observable associated to the entity
     */
    EntityStore.prototype.registerEntity = function (entity, id) {
        if (id !== -1 && this.entitiesObservables[id]) {
            this.entitiesObservables[id].next(entity);
            return this.entitiesObservables[id];
        }
        else {
            var subject = new ReplaySubject(1);
            subject.next(entity);
            if (id !== -1) {
                this.entitiesObservables[id] = subject;
            }
            return subject;
        }
    };
    /**
     * Register an entity subject to a specified id
     * @param {number} id Id used for registration
     * @param {ReplaySubject<DataEntity>} subject Subject to register
     */
    EntityStore.prototype.registerEntitySubject = function (id, subject) {
        if (id !== -1) {
            this.entitiesObservables[id] = subject;
        }
    };
    /**
     * Delete all data for a specific entity id
     * @param {number} id Entity id
     */
    EntityStore.prototype.unregister = function (id) {
        if (this.entitiesObservables[id]) {
            delete this.entitiesObservables[id];
        }
    };
    /**
     * Delete entity from store
     * @param {DataEntity} entity Entity to delete
     */
    EntityStore.prototype.unregisterEntity = function (entity) {
        delete this.entitiesObservables[entity.id];
    };
    /**
     * Returns the observable associated to the specified id
     * @param {number} id Id used in registration
     * @returns {Observable<DataEntity>} Entity subject associated to the id
     */
    EntityStore.prototype.getEntityObservable = function (id, createObservable) {
        if (createObservable === void 0) { createObservable = false; }
        if (this.entitiesObservables[id] && createObservable === false) {
            return this.entitiesObservables[id];
        }
        else {
            var subject = new ReplaySubject(1);
            this.entitiesObservables[id] = subject;
            return subject;
        }
    };
    /**
     * Returns true if entity is defined in store
     * @param {number} entityId Entity id
     * @returns {boolean}
     */
    EntityStore.prototype.isInStore = function (entityId) {
        return !!this.entitiesObservables[entityId];
    };
    return EntityStore;
}());
export { EntityStore };
//# sourceMappingURL=entity-store.class.js.map